# coding: utf-8

"""
    Tada AI API

    API for access Tada resources

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

from pydantic import Field, StrictStr
from typing import Optional
from tada_ai.models.api_model import ApiModel
from tada_ai.models.chunk_options import ChunkOptions
from tada_ai.models.reranker_options import RerankerOptions
from typing import Optional


class SearchQueryInput(ApiModel):
    prompt: StrictStr
    space_id: Optional[StrictStr] = Field(default=None)
    chunk_options: Optional[ChunkOptions] = Field(default=None)
    reranker: Optional[RerankerOptions] = Field(default=None)
