# coding: utf-8

"""
    Tada AI API

    API for access Tada resources

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

from tada_ai.models.api_model import ApiModel


class SearchRecordChunk(ApiModel):
    id: str
    markdown: str = Field(description="The markdown representation of the chunk")
    score: Optional[float] = Field(
        description="The score returned by the reranker. Between 0 and 1"
    )
